import smashggpy.queries.Event_Queries as queries
from smashggpy.util.Logger import Logger
from smashggpy.common.Common import flatten
from smashggpy.util.NetworkInterface import NetworkInterface as NI
from smashggpy.util.ThreadFactory import ThreadFactory


class Event(object):

    def __init__(self, id, name, slug, state, start_at, num_entrants,
                 check_in_buffer, check_in_duration, check_in_enabled,
                 is_online, team_name_allowed, team_management_deadline):
        self.id = id
        self.name = name
        self.slug = slug
        self.state = state
        self.start_at = start_at
        self.num_entrants = num_entrants
        self.check_in_buffer = check_in_buffer
        self.check_in_duration = check_in_duration
        self.check_in_enabled = check_in_enabled
        self.is_online = is_online
        self.team_name_allowed = team_name_allowed
        self.team_management_deadline = team_management_deadline

    @staticmethod
    def get(tournament_slug: str, event_slug: str):
        assert (tournament_slug is not None), "Event.get cannot have None for tournament_slug parameter"
        assert (event_slug is not None), "Event.get cannot have None for event_slug parameter"
        slug = "tournament/{0}/event/{1}".format(tournament_slug, event_slug)
        data = NI.query(queries.get_event_by_slugs, {"slug": slug})

        try:
            base_data = data['data']['event']
            return Event.parse(base_data)
        except AttributeError as e:
            raise Exception("No event data came back for tournament {} and event {}".format(tournament_slug, event_slug))

    @staticmethod
    def get_by_id(id: int):
        assert (id is not None), "Event.get_by_id cannot have None for id parameter"
        data = NI.query(queries.get_event_by_id, {'id': id})

        try:
            base_data = data['data']['event']
            return Event.parse(base_data)
        except AttributeError as e:
            raise Exception("No event data came back for event with id {}".format(id))

    @staticmethod
    def parse(data):
        assert (data is not None), "Event.parse cannot have None for data parameter"
        return Event(
            data['id'],
            data['name'],
            data['slug'],
            data['state'],
            data['startAt'],
            data['numEntrants'],
            data['checkInBuffer'],
            data['checkInDuration'],
            data['checkInEnabled'],
            data['isOnline'],
            data['teamNameAllowed'],
            data['teamManagementDeadline']
        )

    def get_phases(self):
        assert (self.id is not None), "event id cannot be None if calling get_phases"
        Logger.info('Getting Phases for Event: {0}:{1}'.format(self.id, self.name))
        data = NI.query(queries.get_event_phases, {'id': self.id})

        try:
            phases_data = data['data']['event']['phases']
            return [Phase.parse(phase_data) for phase_data in phases_data]
        except AttributeError as e:
            raise Exception("No phase data pulled back for event {} {}".format(self.id, self.name))

    def get_phase_groups(self):
        assert (self.id is not None), "event id cannot be None if calling get_phase_groups"
        Logger.info('Getting Phase Groups for Event: {0}:{1}'.format(self.id, self.name))
        data = NI.query(queries.get_event_phase_groups, {'id': self.id})

        try:
            phase_groups_data = data['data']['event']['phaseGroups']
            return [PhaseGroup.parse(phase_group_data) for phase_group_data in phase_groups_data]
        except AttributeError as e:
            raise Exception("No phase group data pulled back for event {} {}".format(self.id, self.name))

    def get_attendees(self):
        Logger.info('Getting Attendees for Event: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        attendees = flatten([phase_group.get_attendees() for phase_group in phase_groups])
        return attendees

    def get_entrants(self):
        Logger.info('Getting Entrants for Event: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        entrants = flatten([phase_group.get_entrants() for phase_group in phase_groups])
        return entrants

    def get_sets(self):
        Logger.info('Getting Sets for Event: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        sets = flatten([phase_group.get_sets() for phase_group in phase_groups])
        return sets

    def get_incomplete_sets(self):
        Logger.info('Getting Incomplete Sets for Event: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        incomplete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is False:
                incomplete_sets.append(ggset)
        return incomplete_sets

    def get_completed_sets(self):
        Logger.info('Getting Completed Sets for Event: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        complete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is True:
                complete_sets.append(ggset)
        return complete_sets

    # GETTERS
    def get_id(self):
        return self.id

    def get_name(self):
        return self.name

    def get_slug(self):
        return self.slug

    def get_state(self):
        return self.state

    def get_start_at(self):
        return self.start_at

    def get_num_entrants(self):
        return self.num_entrants

    def get_check_in_buffer(self):
        return self.check_in_buffer

    def get_check_in_duration(self):
        return self.check_in_duration

    def get_check_in_enabled(self):
        return self.check_in_enabled

    def get_is_online(self):
        return self.is_online

    def get_team_name_allowed(self):
        return self.team_name_allowed

    def get_team_management_deadline(self):
        return self.team_management_deadline


from smashggpy.models.Phase import Phase
from smashggpy.models.PhaseGroup import PhaseGroup


import smashggpy.queries.Phase_Queries as queries
from smashggpy.util.Logger import Logger
from smashggpy.common.Common import flatten
from smashggpy.util.NetworkInterface import NetworkInterface as NI

class Phase(object):

    def __init__(self, id, name, num_seeds, group_count):
        self.id = id
        self.name = name
        self.num_seeds = num_seeds
        self.group_count = group_count

    @staticmethod
    def get(id: int):
        assert (id is not None), "Phase.get cannot have None for id parameter"
        data = NI.query(queries.phase_by_id, {'id': id})

        try:
            base_data = data['data']['phase']
            return Phase.parse(base_data)
        except AttributeError as e:
            raise Exception("No phase data pulled back for id {}".format(id))

    @staticmethod
    def parse(data):
        assert (data is not None), "Phase.parse cannot have None for data parameter"
        return Phase(
            data['id'],
            data['name'],
            data['numSeeds'],
            data['groupCount']
        )

    def get_phase_groups(self):
        assert (self.id is not None), "phase id cannot be None when calling get_phase_groups"
        Logger.info('Getting Phase Groups for Phase: {0}:{1}'.format(self.id, self.name))
        data = NI.paginated_query(queries.phase_phase_groups, {'id': self.id})
        return [PhaseGroup.parse(phase_group_data) for phase_group_data in data]

    def get_attendees(self):
        Logger.info('Getting Attendees for Phase: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        attendees = flatten([phase_group.get_attendees() for phase_group in phase_groups])
        return attendees

    def get_entrants(self):
        Logger.info('Getting Entrants for Phase: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        entrants = flatten([phase_group.get_entrants() for phase_group in phase_groups])
        return entrants

    def get_sets(self):
        Logger.info('Getting Sets for Phase: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        sets = flatten([phase_group.get_sets() for phase_group in phase_groups])
        return sets

    def get_incomplete_sets(self):
        Logger.info('Getting Incomplete Sets for Phase: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        incomplete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is False:
                incomplete_sets.append(ggset)
        return incomplete_sets

    def get_completed_sets(self):
        Logger.info('Getting Completed Sets for Phase: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        complete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is True:
                complete_sets.append(ggset)
        return complete_sets

    # GETTERS
    def get_id(self):
        return self.id

    def get_name(self):
        return self.name

    def get_num_seeds(self):
        return self.num_seeds

    def get_group_count(self):
        return self.group_count


from smashggpy.models.PhaseGroup import PhaseGroup


import smashggpy.queries.Tournament_Queries as queries

from smashggpy.util.Logger import Logger
from smashggpy.common.Common import flatten
from smashggpy.util.NetworkInterface import NetworkInterface as NI
from smashggpy.models.Venue import Venue
from smashggpy.models.Organizer import Organizer
from smashggpy.common.Exceptions import NoTournamentDataException
from smashggpy.common.Exceptions import NoEventDataException
from smashggpy.common.Exceptions import NoPhaseDataException
from smashggpy.common.Exceptions import NoPhaseGroupDataException


class Tournament(object):

    def __init__(self, id, name, slug, start_time, end_time, timezone, venue, organizer):
        self.id = id
        self.name = name
        self.slug = slug
        self.start_time = start_time
        self.end_time = end_time
        self.timezone = timezone
        self.venue = venue
        self.organizer = organizer

    def __eq__(self, other):
        if other is None:
            return False

        if type(self) != type(other):
            return False

        return hash(self) == hash(other)

    def __hash__(self):
        return hash((self.id, self.name, self.slug, self.start_time, self.end_time, self.timezone, self.venue, self.organizer))

    @staticmethod
    def get(slug: str):
        assert (slug is not None), "Tournament.get must have a slug parameter"
        data = NI.query(queries.get_tournament_by_slug, {'slug': slug})

        if 'errors' in data:
            raise Exception('Error occurred in pulling tournament {}, {}'.format(slug, data['errors']))

        try:
            base_data = data['data']
            print(base_data)

            if base_data['tournament'] is None:
                raise NoTournamentDataException(slug)

            tournament_data = base_data['tournament']
            return Tournament.parse(tournament_data)
        except AttributeError as e:
            raise NoTournamentDataException(slug)


    @staticmethod
    def get_by_id(id: int):
        assert (id is not None), "Tournament.get_by_id must have an id parameter"
        data = NI.query(queries.get_tournament, {'id': id})

        try:
            base_data = data['data']['tournament']
            return Tournament.parse(base_data)
        except AttributeError as e:
            raise NoTournamentDataException(id)

    @staticmethod
    def parse(data):
        assert (data is not None), 'Tournament.parse must have a data parameter'
        assert ('id' in data), 'Tournament.parse must have id in data parameter'
        assert ('name' in data), 'Tournament.parse must have id in data parameter'
        assert ('slug' in data), 'Tournament.parse must have id in data parameter'
        assert ('startAt' in data), 'Tournament.parse must have id in data parameter'
        assert ('endAt' in data), 'Tournament.parse must have id in data parameter'
        assert ('timezone' in data), 'Tournament.parse must have id in data parameter'

        return Tournament(
            data['id'],
            data['name'],
            data['slug'],
            data['startAt'],
            data['endAt'],
            data['timezone'],
            Venue.parse(data),
            Organizer.parse(data)
        )

    def get_events(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_events"
        data = NI.query(queries.get_tournament_events, {'id': self.id})

        try:
            base_data = data['data']['tournament']['events']
            if base_data is None:
                raise NoEventDataException(self.slug)

            return [Event.parse(event_data) for event_data in base_data]
        except AttributeError as e:
            raise NoEventDataException(self.slug)

    def get_phases(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_phases"
        data = NI.query(queries.get_tournament_phases, {'id': self.id})

        try:
            phases = []
            base_events = data['data']['tournament']['events']
            if base_events is None:
                raise NoEventDataException(self.slug)

            for event in base_events:
                try:
                    phase_data = event['phases']
                    if phase_data is None:
                        raise NoPhaseDataException(self.slug)

                    for event_phase in phase_data:
                        phases.append(Phase.parse(event_phase))
                except AttributeError as inner_e:
                    NoPhaseDataException(self.slug)

            return phases
        except AttributeError as e:
            raise NoEventDataException(self.slug)

    def get_phase_groups(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_phase_groups"
        data = NI.query(queries.get_tournament_phase_groups, {'id': self.id})

        try:
            phase_groups = []
            base_events = data['data']['tournament']['events']
            if base_events is None:
                raise NoEventDataException(self.slug)

            for event in base_events:
                try:
                    group_data = event['phaseGroups']
                    if group_data is None:
                        raise NoPhaseGroupDataException(self.slug)

                    for event_phase_groups in group_data:
                        phase_groups.append(PhaseGroup.parse(event_phase_groups))
                except AttributeError as inner_e:
                    NoPhaseGroupDataException(self.slug)

            return phase_groups
        except AttributeError as e:
            raise NoEventDataException(self.slug)

    def get_attendees(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_attendees"
        Logger.info('Getting Attendees for Tournament: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        attendees = flatten([phase_group.get_attendees() for phase_group in phase_groups])
        return attendees

    def get_entrants(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_entrants"
        Logger.info('Getting Entrants for Tournament: {0}:{1}'.format(self.id, self.name))
        Logger.warning('Aggregate queries ')
        phase_groups = self.get_phase_groups()
        entrants = flatten([phase_group.get_entrants() for phase_group in phase_groups])
        return entrants

    def get_sets(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_sets"
        Logger.info('Getting Sets for Tournament: {0}:{1}'.format(self.id, self.name))
        phase_groups = self.get_phase_groups()
        sets = flatten([phase_group.get_sets() for phase_group in phase_groups])
        return sets

    def get_incomplete_sets(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_incomplete_sets"
        Logger.info('Getting Incomplete Sets for Tournament: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        incomplete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is False:
                incomplete_sets.append(ggset)
        return incomplete_sets

    def get_completed_sets(self):
        assert (self.id is not None), "tournament id cannot be None if calling get_completed_sets"
        Logger.info('Getting Completed Sets for Tournament: {0}:{1}'.format(self.id, self.name))
        sets = self.get_sets()
        complete_sets = []
        for ggset in sets:
            if ggset.get_is_completed() is True:
                complete_sets.append(ggset)
        return complete_sets

    # GETTERS
    def get_id(self):
        return self.id

    def get_name(self):
        return self.name

    def get_slug(self):
        return self.slug

    def get_start_time(self):
        return self.start_time

    def get_end_time(self):
        return self.end_time

    def get_timezone(self):
        return self.timezone

    def get_venue(self):
        return self.venue

    def get_organizer(self):
        return self.organizer


from smashggpy.models.Event import Event
from smashggpy.models.Phase import Phase
from smashggpy.models.PhaseGroup import PhaseGroup



class NotInitializedException(Exception):
	def __init__(self, component_name):
		self.message = '{} has not been initialized!'.format(component_name)


class BadTokenException(Exception): 
	def __init__(self, token='No token handed to Exception'):
		self.message = 'Bad Token. Please input a 32 character hexidecimal string.\nGot input "{}"'.format(token)


class NoTournamentDataException(Exception):
	def __init__(self, slug: str):
		self.message = 'No tournament data pulled back for tournament {}'.format(slug)


class NoEventDataException(Exception):
	def __init__(self, slug: str):
		self.message = 'No event data pulled back for tournament {}'.format(slug)


class NoPhaseDataException(Exception):
	def __init__(self, slug: str):
		self.message = 'No phase data pulled back for tournament {}'.format(slug)


class NoPhaseGroupDataException(Exception):
	def __init__(self, slug: str):
		self.message = 'No phase group data pulled back for tournament {}'.format(slug)






class Stream(object):

    def __init__(self, id, event_id, tournament_id, stream_name, num_setups,
                 stream_source, stream_type, stream_type_id, is_online,
                 enabled, follower_count, removes_tasks, stream_status,
                 stream_game, stream_logo):
        self.id = id
        self.event_id = event_id
        self.tournament_id = tournament_id
        self.stream_name = stream_name
        self.num_setups = num_setups
        self.stream_source = stream_source
        self.stream_type = stream_type
        self.stream_type_id = stream_type_id
        self.is_online = is_online
        self.enabled = enabled
        self.follower_count = follower_count
        self.removes_tasks = removes_tasks
        self.stream_status = stream_status
        self.stream_game = stream_game
        self.stream_logo = stream_logo

    @staticmethod
    def parse(data):
        return Stream(
            data['id'],
            data['eventId'],
            data['tournamentId'],
            data['streamName'],
            data['numSetups'],
            data['streamSource'],
            data['streamType'],
            data['streamTypeId'],
            data['isOnline'],
            data['enabled'],
            data['followerCount'],
            data['removesTasks'],
            data['streamStatus'],
            data['streamGame'],
            data['streamLogo'],
        )

    # GETTERS
    def get_id(self):
        return self.id
    
    def get_event_id(self):
        return self.event_id
    
    def get_tournament_id(self):
        return self.tournament_id
    
    def get_stream_name(self):
        return self.stream_name
    
    def get_num_setups(self):
        return self.num_setups
    
    def get_stream_source(self):
        return self.stream_source
    
    def get_stream_type(self):
        return self.stream_type
    
    def get_stream_type_id(self):
        return self.stream_type_id
    
    def get_is_online(self):
        return self.is_online
    
    def get_enabled(self):
        return self.enabled
    
    def get_follower_count(self):
        return self.follower_count
    
    def get_removes_tasks(self):
        return self.removes_tasks
    
    def get_stream_status(self):
        return self.stream_status
    
    def get_stream_game(self):
        return self.stream_game
    
    def get_stream_logo(self):
        return self.stream_logo
    


import os
from pathlib import Path  
from dotenv import load_dotenv

BASE_DIR = Path(os.path.dirname(os.path.realpath(__file__)))
ROOT_DIR = BASE_DIR/'..'/'..'

def dotenv():
	env_path = ROOT_DIR / '.env'
	load_dotenv(dotenv_path=env_path)

def flatten(l: list):
	flat_list = []
	for sublist in l:
		for item in sublist:
			flat_list.append(item)
	return flat_list




class User(object):

    def __init__(self, id, gamer_tag, prefix, color, twitch_stream,
                 twitter_handle, youtube, region, state, country,
                 gamer_tag_changed_at):
        self.id = id
        self.gamer_tag = gamer_tag
        self.prefix = prefix
        self.color = color
        self.twitch_stream = twitch_stream
        self.twitter_handle = twitter_handle
        self.youtube = youtube
        self.region = region
        self.state = state
        self.country = country
        self.gamer_tag_changed_at = gamer_tag_changed_at

    @staticmethod
    def parse(data):
        return User(
            data['id'],
            data['gamerTag'],
            data['prefix'],
            data['color'],
            data['twitchStream'],
            data['twitterHandle'],
            data['youtube'],
            data['region'],
            data['state'],
            data['country'],
            data['gamerTagChangedAt']
        )

    # GETTERS
    def get_id(self):
        return self.id

    def get_gamer_tag(self):
        return self.gamer_tag

    def get_prefix(self):
        return self.prefix

    def get_color(self):
        return self.color

    def get_twitch_stream(self):
        return self.twitch_stream

    def get_twitter_handle(self):
        return self.twitter_handle

    def get_youtube(self):
        return self.youtube

    def get_region(self):
        return self.region

    def get_state(self):
        return self.state

    def get_country(self):
        return self.country

    def get_gamer_tag_changed_at(self):
        return self.gamer_tag_changed_at
